package net.porkbuns.my_story;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import java.io.FileNotFoundException;

//Heavily based on the NotesDBAdapter in the Notepad tutorial
//Database schema:
//quest_id	name	description	completed	status
//quest_id (string): the id of this quest
//name (string): the name of this quest
//description (string): the description of this quest
//completed (string): comma separated list of completed objectives
//status (string): current status of this quest (Active, Completed, or New)
public class QuestDBAdapter {
    public static final String KEY_QUEST_ID="quest_id";
    public static final String KEY_STATUS="status";
    public static final String KEY_OBJECTIVE_ID="objective_id";
    public static final String KEY_="_id";
    public static final String KEY_ROWID="_id";

	 public final int INDEX_QUESTID = 1;
	 
	 public final int INDEX_OBJID = 1;
	 public final int INDEX_OBJ_QUESTID = 2;
	 public final int INDEX_OBJSTATUS = 3;
	 
    
    /**
     * Database creation sql statement
     */
    private static final String ACCEPTEDQUEST_DATABASE_CREATE =
        "create table if not exists acceptedquests (_id integer primary key autoincrement,"
            + "quest_id integer not null);";
    
    private static final String COMPLETEDQUEST_DATABASE_CREATE =
        "create table if not exists completedquests (_id integer primary key autoincrement,"
            + "quest_id integer not null);";
    
    private static final String OBJECTIVE_DATABASE_CREATE =
        "create table if not exists questobjectives (_id integer primary key autoincrement,"
            + "objective_id integer not null, quest_id integer not null, status integer not null);";

    private static final String DATABASE_NAME = "mystory_data";
    private static final String ACCEPTEDQUEST_DATABASE_TABLE = "acceptedquests";
    private static final String COMPLETEDQUEST_DATABASE_TABLE = "completedquests";
    private static final String OBJECTIVE_DATABASE_TABLE = "questobjectives";
    private static final int DATABASE_VERSION = 2;

    private SQLiteDatabase mDb;
    private final Context mCtx;

    /**
     * Constructor - takes the context to allow the database to be opened/created
     * @param ctx the Context within which to work
     */
    public QuestDBAdapter(Context ctx) {
        this.mCtx = ctx;
    }
    
    /**
     * Open the quests database. If it cannot be opened, try to create a new instance of
     * the database. If it cannot be created, throw an exception to signal the failure
     * @return this (self reference, allowing this to be chained in an initialization call)
     * @throws SQLException if the database could be neither opened or created
     */
    public QuestDBAdapter open() throws SQLException {
        try {
            mDb = mCtx.openDatabase(DATABASE_NAME, null);
        } catch (FileNotFoundException e) {
            try {
                mDb =
                    mCtx.createDatabase(DATABASE_NAME, DATABASE_VERSION, 0,
                        null);
                mDb.execSQL(ACCEPTEDQUEST_DATABASE_CREATE);
                mDb.execSQL(COMPLETEDQUEST_DATABASE_CREATE);
                mDb.execSQL(OBJECTIVE_DATABASE_CREATE);
            } catch (FileNotFoundException e1) {
                throw new SQLException("Could not create database");
            }
        }
        return this;
    }

    public void close() {
        mDb.close();
    }

    /**
     * Create a new quest using the values provided. If the quest is successfully created
     * return the new rowId for that quest, otherwise return a -1 to indicate failure.
     * @return rowId or -1 if failed
     */
    public long createQuest(int questId, boolean completed) {
        ContentValues initialValues = new ContentValues();
        initialValues.put(KEY_QUEST_ID, questId);

        if (!completed) {
        	return mDb.insert(ACCEPTEDQUEST_DATABASE_TABLE, null, initialValues);
        }
        else {
        	return mDb.insert(COMPLETEDQUEST_DATABASE_TABLE, null, initialValues);
        }
    }
    
    /**
     * Delete the photo with the given rowId
     * @param rowId id of photo to delete
     * @return true if deleted, false otherwise
     */
    public boolean deleteQuest(int questId, boolean completed) {
    	String tableName;
    	if (!completed) {
    		tableName = ACCEPTEDQUEST_DATABASE_TABLE;
    	}
    	else { 
    		tableName = COMPLETEDQUEST_DATABASE_TABLE;
    	}
        return mDb.delete(tableName, KEY_QUEST_ID + "=" + questId, null) > 0;
    }

    //deletes all quests in the database
    public boolean deleteAllQuests() {
        boolean accepted = mDb.delete(ACCEPTEDQUEST_DATABASE_TABLE, null, null) > 0;
        boolean completed = mDb.delete(COMPLETEDQUEST_DATABASE_TABLE, null, null) > 0;
        
        return accepted && completed;
    }

    /**
     * Return a Cursor over the list of all quests in the database
     * @return Cursor over all quests
     */
    public Cursor fetchAllQuests(boolean completed) {
    	String tableName;
    	if (!completed) {
    		tableName = ACCEPTEDQUEST_DATABASE_TABLE;
    	}
    	else { 
    		tableName = COMPLETEDQUEST_DATABASE_TABLE;
    	}
    
    	Cursor result =  mDb.query(tableName, new String[] {
			KEY_ROWID, KEY_QUEST_ID}, null, null, null, null, null);
    	
    	if ((result.count() == 0) || !result.first()) {
            return null;
        }
        return result;
    }

    
    /**
     * Return a Cursor positioned at the quest that matches the given quest id
     * @param id id of quest to retrieve
     * @return Cursor positioned to matching quest, if found
     * @throws SQLException if quest could not be found/retrieved
     */
    public Cursor fetchQuestGivenId(int id, boolean completed) throws SQLException {
    	String tableName;
    	if (!completed) {
    		tableName = ACCEPTEDQUEST_DATABASE_TABLE;
    	}
    	else { 
    		tableName = COMPLETEDQUEST_DATABASE_TABLE;
    	}
    	
        Cursor result = mDb.query(true, tableName, new String[] {
        		KEY_ROWID, KEY_QUEST_ID}, KEY_QUEST_ID + "=" + id, null, null,
                null, null);
        if ((result.count() == 0) || !result.first()) {
            return null;
        }
        return result;
    }

    /***** OBJECTIVE CODE *****/
    
    /**
     * Create a new quest using the values provided. If the quest is successfully created
     * return the new rowId for that quest, otherwise return a -1 to indicate failure.
     * @return rowId or -1 if failed
     */
    public long createObjective(int objId, int questId, int status) {
        ContentValues initialValues = new ContentValues();
        initialValues.put(KEY_OBJECTIVE_ID, objId);
        initialValues.put(KEY_QUEST_ID, questId);
        initialValues.put(KEY_STATUS, status);
        
        return mDb.insert(OBJECTIVE_DATABASE_TABLE, null, initialValues);
    }

    //deletes all quests in the database
    public boolean deleteAllObjectives() {
        return mDb.delete(OBJECTIVE_DATABASE_TABLE, null, null) > 0;
    }

    /**
     * Return a Cursor over the list of all quests in the database
     * @return Cursor over all quests
     */
    public Cursor fetchAllObjectives() {
    	Cursor result =  mDb.query(OBJECTIVE_DATABASE_TABLE, new String[] {
    			KEY_ROWID, KEY_OBJECTIVE_ID, KEY_QUEST_ID, KEY_STATUS,}, null, null, null, null, null);
    	if ((result.count() == 0) || !result.first()) {
            return null;
        }
        return result;
    }

    /**
     * Return a Cursor positioned at the quest that matches the given quest id
     * @param id id of quest to retrieve
     * @return Cursor positioned to matching quest, if found
     * @throws SQLException if quest could not be found/retrieved
     */
    public Cursor fetchObjectiveGivenQuestId(int questId) throws SQLException {
        Cursor result = mDb.query(true, OBJECTIVE_DATABASE_TABLE, new String[] {
        		KEY_ROWID, KEY_OBJECTIVE_ID, KEY_QUEST_ID, KEY_STATUS}, KEY_QUEST_ID + "=" + questId, null, null,
                null, null);
        if ((result.count() == 0) || !result.first()) {
            return null;
        }
        return result;
    }
    
    /**
     * Return a Cursor positioned at the quest that matches the given quest id
     * @param id id of quest to retrieve
     * @return Cursor positioned to matching quest, if found
     * @throws SQLException if quest could not be found/retrieved
     */
    public Cursor fetchObjectiveGivenQuestIdAndStatus(int questId, int status) throws SQLException {
        Cursor result = mDb.query(true, OBJECTIVE_DATABASE_TABLE, new String[] {
        		KEY_ROWID, KEY_OBJECTIVE_ID, KEY_QUEST_ID, KEY_STATUS}, KEY_QUEST_ID + "=" + questId + " AND " + KEY_STATUS + "=" + status, null, null,
                null, null);
        if ((result.count() == 0) || !result.first()) {
            return null;
        }
        return result;
    }

    /**
     * Update the quest using the details provided. The quest to be updated is specified using
     * the rowId, and it is altered to use the values passed in
     * @return true if the quest was successfully updated, false otherwise
     */
    public boolean updateObjectiveStatus(int objId, int status) {
        ContentValues args = new ContentValues();
        args.put(KEY_STATUS, status);
        return mDb.update(OBJECTIVE_DATABASE_TABLE, args, KEY_OBJECTIVE_ID + "=" + objId, null) > 0;
    }
}
